Scriptname SeeThroughScopesController extends Quest

ReferenceAlias Property PlayerRefAlias Auto Const
ReferenceAlias Property SettingsHolotape Auto Const
ReferenceAlias Property HoldBreathActivator Auto Const
Quest Property MQ102 Auto Const Mandatory
String[] Property AnimationEvents Auto Const
; Effects
Group Effects
	Perk Property SeeThroughScopesHoldBreathActivatorPerk Auto Const
	Spell Property SeeThroughScopesHoldBreathSpell Auto Const
	ImageSpaceModifier Property SeeThroughScopesBlurEffect Auto Const
EndGroup
Group Keywords
	Keyword Property CombatScopeKeyword Auto Const
	Keyword Property HasScope Auto Const
EndGroup

Group GlobalVariables
	GlobalVariable Property SeeThroughScopesBlurStrength Auto Const
	GlobalVariable Property SeeThroughScopesTogglePerspectiveSetting Auto Const
	GlobalVariable Property SeeThroughScopesHoldBreathSetting Auto Const
	GlobalVariable Property SeeThroughScopesWobbleSpeed Auto Const
	GlobalVariable Property SeeThroughScopesWobbleStrength Auto Const
	GlobalVariable Property SeeThroughScopesScriptVersion Auto Const
	GlobalVariable Property SeeThroughScopesModType Auto Const
EndGroup

Group IdleAnimations
	Idle Property STSAimStart Auto Const
	Idle Property STSAimStop Auto Const
	Idle Property STSHoldBreathStart Auto Const
	Idle Property STSHoldBreathStop Auto Const
	Idle Property STSOutOfBreath Auto Const
	Idle Property STSRecover Auto Const
EndGroup
Activator Property SeeThroughScopesHoldBreathActivator Auto Const

;[TAKECOVERVERSION] TakeCoverControl Property Cover Auto Const

; END OF PROPERTIES / START OF Variables
Actor PlayerRef
ObjectReference orHoldBreath = None

; Control variables
Bool bIsInFirstPerson = False
Bool bWasInThirdPerson = False
Bool bWobbleActive = False
Bool bDidFire = False
Bool bPerspectiveToggleCooldown = False
Bool bHoldBreathTriggerActive = False
Bool bIsHoldingBreath = False
Bool bFiredWhileHoldingBreath = False
Bool bIsOutOfBreath = False
Bool bIsUsingCombatScope = False
Bool bHasScopeKeyword = False
Bool bDoShowHoldBreath = False
Bool bDoSwitchPerspective = False
Int iHoldBreathState = 0

Float fOutOfBreathTimer = 2.0
Float fRecoverTimer = 0.5

Float versionNumber = 1.0;
Int iModType = -1;

; Animation Event Processing Variables
Int iNextAnimEvent = 0
Int iHandlingAnimEvent = 0

; Cached version of the global variables
Float fBlurStrength = 0.0
Float fSwayStrength = 0.06
Int iPerspectiveSetting = 1
Int iHoldBreathSetting = 1

; ----------------------------------------
; Constants 
; ----------------------------------------
; Constants for the different settings we have
Int PERSPECTIVE_SETTING_NEVER = 0 Const
Int PERSPECTIVE_SETTING_COMBATSCOPE = 1 Const
Int PERSPECTIVE_SETTING_ALWAYS = 2 Const

Int HOLDBREATH_SETTING_DISABLED = 0 Const
Int HOLDBREATH_SETTING_COMBAT_4X = 1 Const
Int HOLDBREATH_SETTING_COMBAT_ALL = 2 Const
Int HOLDBREATH_SETTING_ALL = 3 Const

; Constants for the different timers we're using
Int TIMER_PERS_TOGGLE_COOLDOWN = 1 Const
Int TIMER_HBT_UPDATE = 2 Const
Int TIMER_OUTOFBREATH = 3 Const
Int TIMER_RECOVERED = 4 Const

; Constants for keeping track of the Hold Breath State
Int HOLDBREATH_STATE_IDLE = 0 Const
Int HOLDBREATH_STATE_HOLDING = 1 Const
Int HOLDBREATH_STATE_OUTOFBREATH = 2 Const
Int HOLDBREATH_STATE_RECOVERING = 3 Const

; Constants for the Animation Event Processing
Int AE_NONE = 0 Const
Int AE_ENTER = 1 Const
Int AE_EXIT = 2 Const
Int AE_HOLDBREATH_START = 3 Const
Int AE_HOLDBREATH_STOP = 4 Const
Int AE_OUTOFBREATH = 5 Const
Int AE_RECOVERING = 6 Const
Int AE_RECOVERED = 7 Const
Int AE_RESET_AIM = 8 Const
Int AE_WEAPON_FIRE = 9 Const

; RELEVANT ANIMATION VARIABLES
; Requires further testing, but the animation variables seem to be persistent
; fSTSWobbleWeight - The intensity of the wobble animation, how much does the camera move
; fSTSWobbleSpeed - The speed at which the wobble animation plays

; Runs as the quest is initialized, so after the references are populated
Event onQuestInit()
	PlayerRef = PlayerRefAlias.getReference() as Actor
	; Fetch a special reference to the player used for the idle animation triggers
	if(PlayerRef == None)
		; dmp("PlayerRef is empty!")
	EndIf
	
	
	if !PlayerRef.hasPerk(SeeThroughScopesHoldBreathActivatorPerk)
		PlayerRef.addPerk(SeeThroughScopesHoldBreathActivatorPerk, false)
	EndIf
	; Register for animation events
	registerAnimationEvents()
	; Register for equip/unequip events to tell whether the player has a Combat Scope equipped
	RegisterForRemoteEvent(PlayerRef,"OnItemEquipped")
	RegisterForRemoteEvent(PlayerRef,"OnItemUnequipped")
	; Update the control booleans
	updateControlBooleans()
	
	bIsInFirstPerson = PlayerRef.GetAnimationVariableBool("IsFirstPerson")
	
	; Check if we can receive the holotape
	checkDidExitFault()
	
	RegisterForRemoteEvent(PlayerRef,"OnPlayerLoadGame")
	; Fetch the mod type from the global val
	iModType = SeeThroughScopesModType.getValue() as Int
EndEvent

Function checkDidExitFault()
	if MQ102.GetStageDone(10) == true
		SetStage(10)
	else
		RegisterForRemoteEvent(MQ102, "OnStageSet")
	endif
EndFunction

Event Actor.OnPlayerLoadGame(Actor akSender)
	; Setup the mod type variable if it wasn't set already (mod was updated from a pre 2.3.3 version)
	if( iModType < 0 )
		; If the ModType reference was not set, then this mod was uninstalled, so terminate the script
		if( !SeeThroughScopesModType )
			terminateMod()
			return
		Else
			; Otherwise, set the correct mod type
			iModType = SeeThroughScopesModType.getValue() as Int
		EndIf
	EndIf
	; check if the mod is still installed, if not, terminate this instance of the script.
	if( iModType == 2 && !Game.isPluginInstalled("3dscopes-replacer.esp") )
		terminateMod()
		return
	ElseIf( iModType == 1 && !Game.isPluginInstalled("3dscopes.esp"))
		terminateMod()
		return
	EndIf
	; check if the script version was updated. If so, perform maintenance (script version only changes with fundamental changes)
	Float compareVersion = SeeThroughScopesScriptVersion.getValue()
	if( compareVersion != versionNumber )
		; Do something to update the script
	EndIf
	
EndEvent

Event Quest.OnStageSet(quest akSendingQuest, int CharGenStage, int StageItem)
	if(akSendingQuest == MQ102 )
		UnregisterForRemoteEvent(MQ102, "OnStageSet")
		CheckDidExitFault()
	EndIf
EndEvent

; Terminates the quest, stops all event listeners
Function terminateMod(Bool bPluginIsActive = True)
	; Unregister all events
	UnregisterForAllEvents()
	; Make sure to remove the hold breath spell
	if( bPluginIsActive )
		Actor Player = Game.GetPlayer();
		SeeThroughScopesBlurEffect.remove()
		; Remove the perk
		Player.removeSpell(SeeThroughScopesHoldBreathSpell)
		Player.removePerk(SeethroughScopesHoldBreathActivatorPerk)
		; Destroy all objects related to this quest
		Player.removeItem(SettingsHolotape.getReference() as Form, -1, true)
		if( orHoldBreath != None )
			orHoldBreath.delete()
		EndIf
	EndIf	
EndFunction

; Called whenever an animation event is unregistred, for instance when we enter/exit power armor
Event OnAnimationEventUnregistered(ObjectReference akSource, String asEventName)
	; we do not really care about the source since we only registered the player for animation events
	; Nevertheless, it might be usefull to check this is the actual source
	Actor source = akSource as Actor
	; if this unregister event didn't come from the player, abort!
	if(source != PlayerRef)
		Return
	EndIf
	; re-register for this event, and update the isInFirstPerson boolean. Corruption from the vertibird will be undone, since the animation graph was reset
	self.RegisterForAnimationEvent(akSource, asEventName)
	bIsInFirstPerson = PlayerRef.GetAnimationVariableBool("IsFirstPerson")
	; Push the animation variables again, as the graph has been reloaded and the values have been reset to default
	if( asEventName == AnimationEvents[0] )
		PlayerRef.setAnimationVariableFloat("fSTSWobbleWeight", SeeThroughScopesWobbleStrength.getValue())
		PlayerRef.setAnimationVariableFloat("fSTSWobbleSpeed", SeeThroughScopesWobbleSpeed.getValue())
	EndIf
EndEvent

; Called whenever a specific animation event is received
Event OnAnimationEvent(ObjectReference akSource, String asEventName)
	Actor source = akSource as Actor
	; Float fDebugId = Utility.getCurrentRealTime()
	; dmp("[" + fDebugId + "] AnimEv: " + asEventName)
	; Check if the animation event originated from the player
	if( source != PlayerRef )
		return
	EndIf
	;Debug.notification(asEventName + " " +Utility.getCurrentRealTime())
	; Handle the different events, first the events to keep track of the camera perspective
	if(asEventName == "FirstPersonInitialized" || asEventName == "PowerFirstPersonInitialized")
		bIsInFirstPerson = True
	ElseIf(asEventName == "TrdP" || asEventName == "PowerThirdPersonInitialized")
		bIsInFirstPerson = False
	ElseIf(asEventName == "sightedStateEnter" && bPerspectiveToggleCooldown == False)
		iNextAnimEvent = AE_ENTER
		; Call the processor, it will check if it should run or not itself
		animEventProcessor()
	ElseIf(asEventName == "sightedStateExit")
		iNextAnimEvent = AE_EXIT
		animEventProcessor()
	ElseIf(iNextAnimEvent == AE_NONE && bWobbleActive)
		If(asEventName == "initiateStart")
			iNextanimEvent = AE_RESET_AIM
			animEventProcessor()
		ElseIf(asEventName == "WeaponFire")
			iNextanimEvent = AE_WEAPON_FIRE
			animEventProcessor()
		EndIf
	EndIf
	; dmp("[" + fDebugId + "] Done: " + asEventName)
EndEvent

Event OnTimer(Int aiTimerId)
	if( aiTimerId == TIMER_PERS_TOGGLE_COOLDOWN )
		bPerspectiveToggleCooldown = False
	ElseIf( aiTimerId == TIMER_HBT_UPDATE )
		updateHoldBreathTrigger()
	ElseIf( aiTimerId == TIMER_OUTOFBREATH && iNextAnimEvent == AE_NONE )
		iNextAnimEvent = AE_RECOVERING
		AnimEventProcessor()
	ElseIf( aiTimerId == TIMER_RECOVERED && iNextAnimEvent == AE_NONE )
		iNextAnimEvent = AE_RECOVERED
		AnimEventProcessor()
	EndIf
EndEvent

Function registerAnimationEvents()
	; Register for all the animation events. If a registration fails, wait for a second, then try again
	Int len = AnimationEvents.length
	Int index = 0
	while( index < len )
		while(!self.RegisterForAnimationEvent(PlayerRef as ObjectReference, AnimationEvents[ index ] ) )
			Utility.wait(1.0)
		EndWhile
		index = index + 1
	EndWhile
EndFunction

; ----------------------------------------
; Actual animation event handler
; ----------------------------------------

; Processes animation events
; Meant to handle rapid (probably accidental) aiming inputs
; NOTE: According to the docs, this script is locked until a function makes an external call
; As such, the function below WILL NOT FUNCTION AS EXPECTED with 'proper' threading
; This functions relies on the fact that as long as we dont make external calls, the script is executed without interrupt
Function AnimEventProcessor()
	; If we're already processing an event, stop
	; TODO: can we merge iHandling and iCurrentAnimEvent?
	if( iHandlingAnimEvent != AE_NONE )
		return
	EndIf
	; dmp("Starting AEP")
	; As long as there are events to handle:
	while( iNextAnimEvent != AE_NONE )
		; Update the currently handling variables
		iHandlingAnimEvent = iNextAnimEvent
		; dmp("Event Number=" + iHandlingAnimEvent)
		; Check which event we should handle
		if( iHandlingAnimEvent == AE_ENTER )
			; dmp("Handling ENTER event")
			; We received an sightedStateEnter event
			; First, make sure we're not in a vats playback		
			;[TAKECOVERVERSION] if( Game.IsVATSPlaybackActive() == False && Cover.getCState() == -1 )
			if( Game.IsVATSPlaybackActive() == False )
				; If we're in third person and we are allowed to switch
				if( !bIsInFirstPerson )
					if( bDoSwitchPerspective )
						bWasInThirdPerson = True
						bIsInFirstPerson = True
						Game.ForceFirstPerson()
					Else
						; Otherwise, do the subgraph fix just in case, to prevent the scope from looking wonky
						; dmp("Doing subgraph fix")
						PlayerRef.playSubgraphAnimation("PowerFirstPersonInitialized")
					EndIf
				EndIf
				
				; If we already were in first person, or we now are,
				; and settings dictate that we should enable hold breath
				if( bIsInFirstPerson )
					if( bDoShowHoldBreath )
						; Enable the hold-breath trigger
						enableHoldBreathTrigger()
					EndIf
					If( bDoShowHoldBreath || (bHasScopeKeyword && bIsUsingCombatScope) || fSwayStrength == 0.0 )
						; Start the wobble/sway animation with a slight delay
						Utility.wait(0.05)
						PlayerRef.playIdle(STSAImStart)
						bWobbleActive = True
						if( fBlurStrength > 0 )
							SeeThroughScopesBlurEffect.apply( fBlurStrength )
						EndIf
					EndIf
				EndIf
			Else
				; If we're in a VATS playback, prevent the scope from going switching its model
				; dmp("Doing subgraph fix")
				PlayerRef.playSubgraphAnimation("PowerFirstPersonInitialized")
			EndIf
		ElseIf( iHandlingAnimEvent == AE_EXIT )
			; dmp("Handling EXIT event")
			; Since Handling can't be None here (while condition) and not ENTER (else-branch of the if above), this must be EXIT
			; Handle the sightedStateExit event			
			; If we were in third person before, and we're no longer aiming, switch back to third person
			if( bWasInThirdPerson && PlayerRef.IsInIronSights() == False )
				Game.ForceThirdPerson()
				bWasInThirdPerson = False
				bIsInFirstPerson = False
				; Enable the perspective switch cooldown
				bPerspectiveToggleCooldown = True
				self.startTimer(0.1, TIMER_PERS_TOGGLE_COOLDOWN)
			EndIf
			; Stop the wobble/sway animation
			PlayerRef.playIdle(STSAimStop)
			bWobbleActive = False
			bDidFire = False
			; If the hold breath trigger was enabled, remove it
			if( bHoldBreathTriggerActive )
				disableHoldBreathTrigger()
			EndIf
			; Remove the blur effect
			SeeThroughScopesBlurEffect.remove()
			; If we're holding our breath, stop it
			if( bIsHoldingBreath )
				stopHoldingBreath(False)
			EndIf
			bFiredWhileHoldingBreath = False
			; If we're out of breath, cancel that
			if( bIsOutOfBreath )
				bIsOutOfBreath = False
				self.cancelTimer(TIMER_OUTOFBREATH)
				self.cancelTimer(TIMER_RECOVERED)
			EndIf
		ElseIf( iHandlingAnimEvent == AE_HOLDBREATH_START )
			; Handle the 'Start holding breath' event
			startHoldingBreath()
;			iHoldBreathState = HOLDBREATH_STATE_HOLDING
		ElseIf( iHandlingAnimEvent == AE_HOLDBREATH_STOP )
			; Handle the 'Stop Holding Breath' event
			stopHoldingBreath()
		ElseIf( iHandlingAnimEvent == AE_OUTOFBREATH )
			; Handle the out of breath event
			bIsOutOfBreath = True
			StopHoldingBreath(False)
;			iHoldBreathState = HOLDBREATH_STATE_OUTOFBREATH
			PlayerRef.playIdle(STSOutOfBreath)
			self.startTimer(fOutOfBreathTimer, TIMER_OUTOFBREATH)
			disableHoldBreathTrigger()
		ElseIf( iHandlingAnimEvent == AE_RECOVERING )
			; Handle the 'start recovering from out of breath' event
			PlayerRef.playIdle(STSRecover)
			self.startTimer(fRecoverTimer, TIMER_RECOVERED)
;			iHoldBreathState = HOLDBREATH_STATE_RECOVERING
		ElseIf( iHandlingAnimEvent == AE_RECOVERED )
			; Handle the 'recovered from out of breath' event
			bIsOutOfBreath = False
			enableHoldBreathTrigger()
;			iHoldBreathState = HOLDBREATH_STATE_IDLE
		ElseIf( iHandlingAnimEvent == AE_RESET_AIM )
			; Aim state was reset, probably due to firing, reset it
			PlayerRef.playIdle(STSAimStart)
			bDidFire = False
			if( bIsHoldingBreath )
				PlayerRef.playIdle(STSHoldBreathStart)
			ElseIf(bFiredWhileHoldingBreath)
				startHoldingBreath()
			EndIf
		ElseIf( iHandlingAnimEvent == AE_WEAPON_FIRE )
			PlayerRef.playIdle(STSAimStop)
			bDidFire = True
			if( bIsHoldingBreath )
				stopHoldingBreath(False)
				bFiredWhileHoldingBreath = True
			EndIf
		EndIf
		; After we're done handling the event, update the control variables
		; if the current state is equal to the next queued event, then nothing more needs to be done
		if( iHandlingAnimEvent == iNextAnimEvent )
			iNextAnimEvent = AE_NONE
			iHandlingAnimEvent = AE_NONE
		EndIf
	EndWhile
	; dmp("AEP Done")
EndFunction

; ----------------------------------------
; Hold Breath Trigger related functions
; ----------------------------------------

; Called whenever we hit the activate button to toggle the hold breath feature
Function activateHoldBreath(ObjectReference akTargetRef)
	; If we're out of breath, ignore the input (Note: the activator will be hidden ASAP)
	if bIsOutOfBreath || iNextAnimEvent != AE_NONE || bDidFire
		Return
	EndIf
	
	if((!bHoldBreathTriggerActive || !bDoShowHoldBreath) && akTargetRef.getBaseObject().getFormId() == (SeeThroughScopesHoldBreathActivator as Form).getFormId() )
		akTargetRef.delete()
		return
	EndIf
	
	if !bIsHoldingBreath
		iNextAnimEvent = AE_HOLDBREATH_START
	else
		iNextAnimEvent = AE_HOLDBREATH_STOP
	EndIf
	AnimEventProcessor()
EndFunction

Function startHoldingBreath(Bool bAnimationTrigger = True)
	bIsHoldingBreath = True
	PlayerRef.AddSpell(SeeThroughScopesHoldBreathSpell, False)
	PlayerRef.playIdle(STSHoldBreathStart)
EndFunction

Function stopHoldingBreath(Bool bAnimationTrigger = True)
	if( !bIsHoldingBreath )
		return
	EndIf
	PlayerRef.RemoveSpell(SeeThroughScopesHoldBreathSpell)
	if( bAnimationTrigger )
		PlayerRef.playIdle(STSHoldBreathStop)
	EndIf
	bIsHoldingBreath = False
;	iHoldBreathState = HOLDBREATH_STATE_IDLE
EndFunction

; Called to trigger an 'out-of-breath' event
Function triggerOutOfBreath()
	; dmp("Triggering out of breath")
	if( iNextAnimEvent != AE_NONE || !bIsHoldingBreath )
		Return
	EndIf
	iNextAnimEvent = AE_OUTOFBREATH
	animEventProcessor()
EndFunction

; Called to enable the Hold Breath trigger box
Function enableHoldBreathTrigger()
	if( orHoldBreath == None )
		orHoldBreath = PlayerRef.placeAtMe(SeeThroughScopesHoldBreathActivator as Form, 1, True, False, False)
		orHoldBreath.setScale(3.0)
		orHoldBreath.moveTo(PlayerRef, 0, 0, -128, false)
		orHoldBreath.enableNoWait()
	EndIf	
	bHoldBreathTriggerActive = True
	self.startTimer(0.20, TIMER_HBT_UPDATE)
	Utility.setINIFloat("fCrosshairAlphaPercent:Interface", 0.0)	
EndFunction

; Called to update the position of the Hold Breath trigger box
Function updateHoldBreathTrigger()
	if( !bHoldBreathTriggerActive || !bDoShowHoldBreath)
		orHoldBreath.delete()
		orHoldBreath = None
		return
	EndIf
	orHoldBreath.moveTo(PlayerRef, 0, 0, -128, false)
	self.startTimer(0.20, TIMER_HBT_UPDATE)
EndFunction

; Called to hide the Hold Breath trigger box
Function disableHoldBreathTrigger()
	orHoldBreath.disableNoWait(false)
	orHoldBreath.delete();
	orHoldBreath = None
	bHoldBreathTriggerActive = False
	self.cancelTimer(TIMER_HBT_UPDATE)
	Utility.setINIFloat("fCrosshairAlphaPercent:Interface", 100.0)
EndFunction

; ----------------------------------------
; Variable Cache related functions
; ----------------------------------------

; Updates the values of the cached global variables. Updates them all
Function updateCachedGlobalVars()
	updateBlurStrength()
	updateSwayStrength()
	updateSwaySpeed()
	updatePerspectiveSetting()
	updateHoldBreathSetting()

	fBlurStrength = SeeThroughScopesBlurStrength.getValue()
	iPerspectiveSetting = SeeThroughScopesTogglePerspectiveSetting.getValue() as Int
	; dmp("Global var cache updated")
	; dmp("Blur Strength: " + fBlurStrength)
	; dmp("Perspective: " + iPerspectiveSetting)
EndFunction

; Resets all settings to their default
Function resetSettingsToDefault()
	SeeThroughScopesBlurStrength.setValue(0.0)
	SeeThroughScopesHoldBreathSetting.setValue(1.0)
	SeeThroughScopesTogglePerspectiveSetting.setValue(1.0)
	SeeThroughScopesHoldBreathSetting.setValue(1.0)
	SeeThroughScopesWobbleStrength.setValue(0.06)
	SeeThroughScopesWobbleSpeed.setValue(1.0)
	
	updateCachedGlobalVars()
EndFunction

; Update the blur strength
Function updateBlurStrength()
	fBlurStrength = SeeThroughScopesBlurStrength.getValue()
EndFunction

; Update the idle sway strength when aiming
Function updateSwayStrength()
	fSwayStrength = SeeThroughScopesWobbleStrength.getValue()
	PlayerRef.setAnimationVariableFloat("fSTSWobbleWeight", SeeThroughScopesWobbleStrength.getValue())
	updateDoHoldBreathBoolean()
	; dmp("fSTSWobbleWeight=" + PlayerRef.getAnimationVariableFloat("fSTSWobbleWeight"))
EndFunction

; Update the idle sway speed when aiming
Function updateSwaySpeed()
	; fSwaySpeed = SeeThroughScopesWobbleSpeed.getValue()
	PlayerRef.setAnimationVariableFloat("fSTSWobbleSpeed", SeeThroughScopesWobbleSpeed.getValue())
	; dmp("fSTSWobbleSpeed=" + PlayerRef.getAnimationVariableFloat("fSTSWobbleSpeed"))
EndFunction

; Update the auto-toggle perspective setting
Function updatePerspectiveSetting()
	iPerspectiveSetting = limitToInt(SeeThroughScopesTogglePerspectiveSetting.getValue(),PERSPECTIVE_SETTING_NEVER,PERSPECTIVE_SETTING_ALWAYS)
	; dmp("iPerspectiveSetting=" + iPerspectiveSetting)
	updateDoSwitchPerspectiveBoolean()
EndFunction

; Update the hold breath setting
Function updateHoldBreathSetting()
	iHoldBreathSetting = limitToInt(SeeThroughScopesHoldBreathSetting.getValue(), HOLDBREATH_SETTING_DISABLED, HOLDBREATH_SETTING_ALL)
	; dmp("iHoldBreathSetting=" + iHoldBreathSetting)
	updateDoHoldBreathBoolean()
EndFunction

Int Function limitToInt(Float value, Int min, Int max)
	return Math.min(Math.max(value,min),max) as Int
EndFunction 

; Updates the values of the cached keyword statuses: if the player is using a weapon with a combat scope
Event Actor.OnItemEquipped(Actor akActor, Form akBaseObject, ObjectReference akReference)
	if akBaseObject as Weapon
		updateControlBooleans()
	EndIf
EndEvent

; Updates the values of the cached keyword statuses: if the player is using a weapon with a combat scope
Event Actor.OnItemUnequipped(Actor akActor, Form akBaseObject, ObjectReference akReference)
	if akBaseObject as Weapon
		updateControlBooleans()
	EndIf
EndEvent

; Update the bool variables that control whether we enable hold breath and if we switch perspective
Function updateControlBooleans()
	bHasScopeKeyword = PlayerRef.wornHasKeyword( HasScope )
	bIsUsingCombatScope = PlayerRef.wornHasKeyword( CombatScopeKeyword )
	
	updateDoHoldBreathBoolean()
	updateDoSwitchPerspectiveBoolean()
EndFunction

; Update the hold breath control boolean
Function updateDoHoldBreathBoolean()
	bDoShowHoldBreath = False
	If( fSwayStrength != 0.0 )
		If( iHoldBreathSetting == HOLDBREATH_SETTING_COMBAT_4X && bHasScopeKeyword && bIsUsingCombatScope)
			bDoShowHoldBreath = True
		ElseIf( iHoldBreathSetting == HOLDBREATH_SETTING_COMBAT_ALL && bIsUsingCombatScope)
			bDoShowHoldBreath = True
		ElseIf( iHoldBreathSetting == HOLDBREATH_SETTING_ALL )
			bDoShowHoldBreath = True
		EndIf
	EndIf
	; dmp("bDoshowHoldBreath=" + bDoShowHoldBreath)
EndFunction

; Update the switch perspective control boolean
Function updateDoSwitchPerspectiveBoolean()
	bDoSwitchPerspective = False
	if( iPerspectiveSetting == PERSPECTIVE_SETTING_ALWAYS )
		bDoSwitchPerspective = True
	ElseIf( iPerspectiveSetting == PERSPECTIVE_SETTING_COMBATSCOPE && bIsUsingCombatScope )
		bDoSwitchPerspective = True
	EndIf
EndFunction

; ----------------------------------------
; Debug related functions
; ----------------------------------------

Function dmp(String str)
	Debug.Trace("STSC|" + str)
EndFunction